﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using java.util;
using com.ihr.xbrl.om;
using com.ihr.xbrl.om.xdt;
using java.io;
using java.net;
using com.ihr.xbrl.om.instance;
using com.ihr.xbrl.om.formula;
using javax.xml.@namespace;

namespace DimensionsInFactItems
{
    class DimensionsInFactItems
    {
        static void Main(string[] args)
        {
            System.Console.WriteLine("Starting...");
            // the main purpose of this demo is to show how to obtain dimensions in the DTS.

            // we will:
            // 1. open a taxonomy that contains dimensions.
            // 2. validate the document against the XBRL 2.1 and Dimensions specifications.
            // 3. loop over the dimensions and show information about it.

            // Let's define the properties of the DTSContainer first. By default, the DTSContainer will perform only XBRL 2.1 validation.
            // we can easily configure the processor to perform XBRL 2.1 and Dimensions validation.

            Properties dtsProps = new Properties();
            dtsProps.setProperty(DTSContainer.PROCESSORS_SEQUENCE,
                XBRLCoreProcessor.PROCESSORNAME + "," + XDTProcessor.PROCESSORNAME);

            // Hint: adding more processors to the sequence is possible (Formulas) it is also possible creating custom
            // processors for validation of project specific things that are not part of the XBRL and cannot be part
            // of the Formulas, Dimensions or core XBRL 2.1 and attach them to the sequence.

            // Now lets create the URL of the taxonomy file with dimensions
            // change this if the file is in another directory.
            // The selected instance document is using COREP taxonomies 1.3.
            File fInstance = new File("TaxonomyFiles\\CA_instance.xml");
            URI uInstance = fInstance.toURI(); // it is easier this way because URIs are more difficult to write when they are pointing to local files.

            // create the DTSContainer using indicated properties
            DTSContainer myDTS = DTSContainer.newEmptyContainer(dtsProps);

            System.Console.WriteLine("Press any key to start loading the instance...");
            System.Console.ReadKey(true);
            System.Console.WriteLine("Loading... this may take a while...");

            // note this cast is not safe and will fail if the document pointed to by the URL is a linkbase or taxonomy. But this is a demo.
            XBRLInstance myInstance = (XBRLInstance)myDTS.load(uInstance);

            System.Console.WriteLine("Loaded; Press any key to start validating the instance...");
            System.Console.ReadKey(true);
            System.Console.WriteLine("Validating...");

            // Now validate the instance document. This will loop all processor indicated in the PROCESSORS_SEQUENCE property
            // and will collect everything found (Errors, Inconsistencies, Assertions, Formula, ... into the XBRLValidationReport
            XBRLValidationReport report = myDTS.validate();

            System.Console.Write("Validated ");

            // in case of errors. we cannot continue. this call will thow the first error as an XBRLValidationExcecption and will do
            // nothing if the document is clean. Note we don't mind about inconsistencies at this stage. All of them are collected
            // inside the XBRLValidationReport
            report.throwExceptions();

            System.Console.WriteLine("s; Press any key to start exploring the instance content...");
            System.Console.ReadKey(true);
            System.Console.WriteLine("Exploring...");

            // obtain the instance of the dimensions processor used during dimensions validation. 
            // (or creates a new one if it were not used) this is the safe way to obtain the instance
            // of the XDTProcessor.
            XDTProcessor xdt = XDTProcessor.create(myDTS);

            // all this is only needed for method 2.
            XDTInfosetConsumer ic = new XDTInfosetConsumer(xdt);

            // you could also do. 
            // XDTProcessor xdt2 = (XDTProcessor)myDTS.getProcessor(XDTProcessor.PROCESSORNAME);
            // and check if the return value is null. It is null if the processor has not been created before.
            // this is the reason the previous method always success.

            // now let's explore the report content. Loop over all existing facts
            Iterator iterFacts = myInstance.iterator();
            while (iterFacts.hasNext())
            {
                XBRLFact fact = (XBRLFact)iterFacts.next();
                System.Console.WriteLine("Fact " + printQName(fact.getNodeName()));

                // only items may have dimensions
                if (fact is XBRLFactItem)
                {
                    // method 1: lets use the Formula Processor Class FContext
                    /*
                    FContext fc = new FContext(fact.getElementDefinition(),((XBRLFactItem)fact).getContext());

                    if (fc.getDimensions().size() > 0)
                    {
                        foreach (QName qDimName in fc.getDimensions().toArray())
                        {
                            System.Console.Write(" contains dimension "+printQName(qDimName));
                            XMLFragment fr = fc.getDimension(qDimName);

                            // Dimensions can be explicit or typed. The dimension content is different

                            if (FContext.isExplicitDimensionContainer(fr))
                            {
                                System.Console.WriteLine(" explitit, and dimension value is " + printQName(fr.getQNameValue()));
                            }
                            else if (FContext.isTypedDimensionContainer(fr))
                            {
                                System.Console.WriteLine(" typed, and dimension value is " + fr.getChild(0).toString());
                            }
                        }                        
                    }
                    else
                    {
                        System.Console.WriteLine(" contains no dimensions.");
                    }
                    */

                    // method 2: lets use the Dimensions Processor classes

                    // the instance document is already valid, so do not perform validation
                    // this is the reason we put 'false' in the third parameter
                    XDTFactForApplications xFact = new XDTFactForApplications((XBRLFactItem)fact, xdt, ic, false);

                    Iterator iterSeg = xFact.getDimensions(XDTHypercubeUse.SEGMENT);
                    loopDimensions(iterSeg, xFact);

                    Iterator iterSce = xFact.getDimensions(XDTHypercubeUse.SCENARIO);
                    loopDimensions(iterSce, xFact);
                }
                else
                {
                    System.Console.WriteLine(" only items may contain dimensions");
                }
            }

            System.Console.WriteLine("Finished; Press any key to exit...");
            System.Console.ReadKey(true);
        }

        static string printQName(QName q)
        {
            return (q.getPrefix().Length == 0 ? "" : q.getPrefix() + ":") + q.getLocalPart();
        }

        static void loopDimensions(Iterator iter, XDTFactForApplications fact)
        {
            while (iter.hasNext())
            {
                QName qDimensionName = (QName)iter.next();
                object dimValue = fact.getDimensionValue(qDimensionName);
                if (dimValue is QName)
                {
                    System.Console.WriteLine(" contains dimension " + printQName(qDimensionName) + " explitit, and dimension value is " + printQName((QName)dimValue));
                }
                if (dimValue is XMLFragment)
                {
                    System.Console.WriteLine(" contains dimension " + printQName(qDimensionName) + " typed, and dimension value is " + ((XMLFragment)dimValue).toString());
                }
            }
        }

    }
}
