﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using java.util;
using com.ihr.xbrl.om.xdt;
using com.ihr.xbrl.om;
using java.net;
using com.ihr.xbrl.om.instance;
using javax.xml.@namespace;
using com.ihr.xbrl.om.taxonomy;
using net.sf.saxon.value;

namespace SampleCreateXBRLReport
{
    class Program
    {
        // this sample application shows how to create an XBRL report from scratch.
        //
        // The main steps are:
        // 1. Open the DTS (Taxonomy) from the entry point (or entry points if more than one are required)
        // 2. Create a new Empty XBRL Report
        // 3. Add fact values to the XBRL report
        // 4. Validate the new XBRL Report against the specifications
        // 5. Save the XBRL report to disk
        static void Main(string[] args)
        {

            // Configure the XBRL processor according to needs. In this case,
            // the taxonomy contains dimensions so let's add the XDTProcessor
            // to the list of processors in the validation sequence.
            Properties props = new Properties();

            String processorsSequence = XBRLCoreProcessor.PROCESSORNAME+","+XDTProcessor.PROCESSORNAME;
            props.setProperty(DTSContainer.PROCESSORS_SEQUENCE, processorsSequence);

            // for more information about configuration parameters visit the DTSContainer documentation at
            // http://www.reportingstandard.com/apidoc/com/ihr/xbrl/om/DTSContainer.html

            // create a new empty container
            DTSContainer dts = DTSContainer.newEmptyContainer(props);

            // 1. Open the DTS (Taxonomy) from the folowing two entry points. Note by default the taxonomy catalog
            // will be used so no internet access is needed and no files will be opened from the Internet
            URI lbPres = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/pre_ias_1_2011-03-25_role-110000.xml");
            URI lbLabels = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/labels/lab_ifrs-en_2011-03-25.xml");
            URI lbPre = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/pre_ias_1_2011-03-25_role-220000.xml");
            URI lbPre2 = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/pre_ias_1_2011-03-25_role-110000.xml");
            URI lbCal = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/cal_ias_1_2011-03-25_role-220000.xml");
            dts.load(new URI[] { lbPres, lbLabels, lbPre, lbPre2, lbCal });

            // we will be using the IFRS namespace so delcare it
            string IFRS = "http://xbrl.ifrs.org/taxonomy/2011-03-25/ifrs";

            // 2. Create a new Empty XBRL Report
            XBRLInstance report = new XBRLInstance(dts);

            // 3. Add fact values to the XBRL report

            // Create two XBRL context 
            XBRLEntity company = new XBRLEntity(dts,"http://www.xbrl.org/scheme","Sample Company",null);
            XBRLPeriod periodD = new XBRLPeriod(dts, "2010-01-01", "2010-12-31");
            XBRLContext ctxD = new XBRLContext(dts, company, periodD, null);

            XBRLPeriod periodI = new XBRLPeriod(dts, "2010-12-31");
            XBRLContext ctxI = new XBRLContext(dts, company, periodI, null);

            // add the first item
            XBRLItem NameOfReportingEntityOrOtherMeansOfIdentificationItem = (XBRLItem)dts.getConcept(new QName(IFRS,"NameOfReportingEntityOrOtherMeansOfIdentification"));
            XBRLFactItem f1 = new XBRLFactNonNumeric(report, ctxD, NameOfReportingEntityOrOtherMeansOfIdentificationItem);
            f1.setValue(new StringValue("Sample Company"));

            // add other items

            XBRLItem PeriodCoveredByFinancialStatementsItem = (XBRLItem)dts.getConcept(new QName(IFRS, "PeriodCoveredByFinancialStatements"));
            XBRLFactItem f2 = new XBRLFactNonNumeric(report, ctxD, PeriodCoveredByFinancialStatementsItem);
            f2.setValue(new StringValue("Year 2010"));

            XBRLItem DescriptionOfNatureOfFinancialStatementsItem = (XBRLItem)dts.getConcept(new QName(IFRS, "DescriptionOfNatureOfFinancialStatements"));
            XBRLFactItem f3 = new XBRLFactNonNumeric(report, ctxD, DescriptionOfNatureOfFinancialStatementsItem);
            f3.setValue(new StringValue("Balance Sheet"));

            Vector numeratorUnits = new Vector();
            numeratorUnits.add(new QName("http://www.xbrl.org/2003/iso4217","EUR","iso4217"));
            XBRLUnit unit = new XBRLUnit(dts, numeratorUnits, null);

            XBRLItem PropertyPlantAndEquipmentItem = (XBRLItem)dts.getConcept(new QName(IFRS, "PropertyPlantAndEquipment"));
            XBRLFactNumeric f4 = new XBRLFactNumeric(report, ctxI, PropertyPlantAndEquipmentItem, unit, new XBRLNumericValue(null, new DecimalValue(12), XBRLNumber.__Fields.INF));

            XBRLItem InvestmentPropertyItem = (XBRLItem)dts.getConcept(new QName(IFRS, "InvestmentProperty"));
            XBRLFactNumeric f5 = new XBRLFactNumeric(report, ctxI, InvestmentPropertyItem, unit, new XBRLNumericValue(null, new DecimalValue(34), XBRLNumber.__Fields.INF));

            XBRLItem GoodwillItem = (XBRLItem)dts.getConcept(new QName(IFRS, "Goodwill"));
            XBRLFactNumeric f6 = new XBRLFactNumeric(report, ctxI, GoodwillItem, unit, new XBRLNumericValue(null, new DecimalValue(56), XBRLNumber.__Fields.INF));

            // 4. Validate the new XBRL Report against the specifications

            XBRLCoreProcessor proc = XBRLCoreProcessor.create(dts);
            XBRLValidationReport valReport = proc.validate(report);

            // in case of errors just raise them.
            valReport.throwExceptions();
            valReport.inconsistenciesAsExceptions();

            // 5. Save the XBRL report to disk

            URI docName = new URI("sampleReport.xbrl");
            report.setURI(docName);
            report.save(true);
        }
    }
}
